#!/bin/bash
set -e
LIGHT_GREEN='\x1B[1;32m'
NC='\x1B[0m' # No Color
RED='\x1B[1;31m'

CLOUDIFY_RPM_FILENAME="cloudify-centos7.rpm"
CLOUDIFY_DOWNLOAD_URL="http://xap-cloud.s3.amazonaws.com/12.0/${CLOUDIFY_RPM_FILENAME}"
CLOUDIFY_DOWNLOAD_FOLDER="cloudify"
CLOUDIFY_RPM_PACKAGE_NAME="cloudify-centos-Core-cli*"
BLUEPRINT_FILENAME="blueprint.tar"
BLUEPRINT_DOWNLOAD_URL="http://xap-cloud.s3.amazonaws.com/12.0/${BLUEPRINT_FILENAME}"
BLUEPRINT_DOWNLOAD_FOLDER="blueprint"

if [ ! -n "${TASK_THREAD_POOL}" ]; then
    TASK_THREAD_POOL=4
fi

function get_file_full_path {
    echo "$(cd "$(dirname "$1")"; pwd)/$(basename "$1")"
}

LOG_FILE_NAME=$(get_file_full_path "xap-cloud-$(date +%Y.%m.%d-%H.%M.%S).log")

function loginfo {
    echo -e $(date +"%Y-%m-%d %H:%M:%S") INFO: $@ | tee -a ${LOG_FILE_NAME}
}

function logerror {
    echo -e $(date +"%Y-%m-%d %H:%M:%S") ERROR: $@ | tee -a ${LOG_FILE_NAME}
}

function handle_error {
    logerror "${RED}$1 Please refer to the log file for more information${NC}"
    loginfo "Log file: ${LOG_FILE_NAME}"
    exit 1
}

function cloudify_is_installed {
    sudo rpm -qa | grep ${CLOUDIFY_RPM_PACKAGE_NAME} | wc -l
}

function download_cloudify {
    mkdir ${CLOUDIFY_DOWNLOAD_FOLDER}
    wget -O ${CLOUDIFY_DOWNLOAD_FOLDER}/${CLOUDIFY_RPM_FILENAME} ${CLOUDIFY_DOWNLOAD_URL} >> ${LOG_FILE_NAME} 2>&1
}

function download_blueprint {
    mkdir ${BLUEPRINT_DOWNLOAD_FOLDER}
    wget -O ${BLUEPRINT_DOWNLOAD_FOLDER}/${BLUEPRINT_FILENAME} ${BLUEPRINT_DOWNLOAD_URL} >> ${LOG_FILE_NAME} 2>&1
}

function install_cloudify_if_needed {
    loginfo "Checking if Cloudify is installed"
    if [ "`cloudify_is_installed`" -eq 1 ]; then
        loginfo "Cloudify is installed..."
    else
        if [ ! -f "${CLOUDIFY_DOWNLOAD_FOLDER}/${CLOUDIFY_RPM_FILENAME}" ]; then
            loginfo "Downloading Cloudify installer (rpm)"
            download_cloudify || (rm -rf "${CLOUDIFY_DOWNLOAD_FOLDER}" && handle_error "Failed to download Cloudify.")
        fi

        loginfo "Installing Cloudify CLI"
        sudo rpm -i ${CLOUDIFY_DOWNLOAD_FOLDER}/${CLOUDIFY_RPM_FILENAME} >> ${LOG_FILE_NAME} 2>&1 || handle_error "Failed to install Cloudify CLI."
    fi
}

function install_blueprint_if_needed {
    loginfo "Checking if blueprint is downloaded"
    if [ -d "${BLUEPRINT_DOWNLOAD_FOLDER}" ]; then
        loginfo "Blueprint is already exist"
    else
        loginfo "Downloading and installing XAP blueprint"
        download_blueprint || (rm -rf "${BLUEPRINT_DOWNLOAD_FOLDER}" && handle_error "Failed to download blueprint.")
        echo "Unzipping blueprint" >> ${LOG_FILE_NAME} 2>&1
        tar -xvf ${BLUEPRINT_DOWNLOAD_FOLDER}/${BLUEPRINT_FILENAME} -C ${BLUEPRINT_DOWNLOAD_FOLDER} >> ${LOG_FILE_NAME} 2>&1
    fi
}

function filter_cloudify_logs {
awk '
!/preparing fabric environment/ && !/environment prepared successfully/ && /^[0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9] [0-9][0-9]:[0-5][0-9]:[0-5][0-9] .* \[.*(start|configure|stop|delete)\] INFO.*/ { printf $1" "$2" "$7" "$6" "; for (i=8; i<NF; i++) printf $i" "; print $NF }
!/preparing fabric environment/ && !/environment prepared successfully/ && /^\[.*\] out: [0-9][0-9][0-9][0-9]-[0-9][0-9]-[0-9][0-9] [0-9][0-9]:[0-5][0-9]:[0-5][0-9] .* \[.*(start|configure|stop|delete)\] INFO.*/ { printf $3" "$4" "$9" "$8" "; for (i=10; i<NF; i++) printf $i" "; print $NF }'
}

function update_node_counts {
    python update_hosts.py $@
}

function install_xap {
    rm -rf ${BLUEPRINT_DOWNLOAD_FOLDER}/xap-hosts.yaml*

    install_cloudify_if_needed

    install_blueprint_if_needed

    cp xap-hosts.yaml ${BLUEPRINT_DOWNLOAD_FOLDER}/xap-hosts.yaml

    if [ -f "pre-install.sh" ]; then
        yes | cp pre-install.sh ${BLUEPRINT_DOWNLOAD_FOLDER}/xap-scripts/pre-install.sh
    fi

    if [ -f "pre-start.sh" ]; then
        yes | cp pre-start.sh ${BLUEPRINT_DOWNLOAD_FOLDER}/xap-scripts/pre-start.sh
    fi

    cd ${BLUEPRINT_DOWNLOAD_FOLDER}

    loginfo "Activating virtual env"
    source /opt/cfy/env/bin/activate

    update_node_counts xap-hosts.yaml xap-blueprint.yaml

    loginfo "Initializing ..."
    cfy local init --install-plugins -p xap-blueprint.yaml -i ../xap-cloud.yaml >> ${LOG_FILE_NAME} 2>&1 || handle_error "Initialization failed."

    loginfo "Installing ..."
    cfy local execute --task-thread-pool-size=${TASK_THREAD_POOL} --workflow install 2>&1 | tee -a ${LOG_FILE_NAME} | filter_cloudify_logs || handle_error "Installation failed."
     #-v --debug

    if [ "${PIPESTATUS[0]}" -ne 0 ]; then
        handle_error "Failed to install XAP."
    else
        loginfo "${LIGHT_GREEN}XAP is installed successfully${NC}"
    fi

    OUTPUT=`cfy local outputs`
    regex='(https?)://[-A-Za-z0-9\+&@#/%?=~_|!:,.;]*[-A-Za-z0-9\+&@#/%=~_|]'

    if [[ ${OUTPUT} =~ $regex ]]; then
      WEBUI_URL=${BASH_REMATCH} ;
      loginfo "${LIGHT_GREEN}WebUI is available at ${WEBUI_URL}${NC}"
    fi
}

function uninstall_xap {
    cd ${BLUEPRINT_DOWNLOAD_FOLDER}

    loginfo "Activating virtual env"
    source /opt/cfy/env/bin/activate

    loginfo "Executing uninstall workflow"
    cfy local execute --task-thread-pool-size=${TASK_THREAD_POOL} --workflow uninstall 2>&1 | tee -a ${LOG_FILE_NAME} | filter_cloudify_logs

    if [ "${PIPESTATUS[0]}" -ne 0 ]; then
        handle_error "Failed to uninstall XAP."
    else
        loginfo "${LIGHT_GREEN}XAP is uninstalled successfully${NC}"
    fi
}

case $1 in
    "install")
        install_xap;;
    "uninstall")
        uninstall_xap;;
    "")
        echo "Valid options are: install, uninstall";;
    *)
        echo "Invalid option $1"
        echo "Valid options are: install, uninstall";;
esac
